# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64

from xdrlib3 import Packer, Unpacker

from .hash import Hash
from .uint32 import Uint32

__all__ = ["ArchivalProofNode"]


class ArchivalProofNode:
    """
    XDR Source Code::

        struct ArchivalProofNode
        {
            uint32 index;
            Hash hash;
        };
    """

    def __init__(
        self,
        index: Uint32,
        hash: Hash,
    ) -> None:
        self.index = index
        self.hash = hash

    def pack(self, packer: Packer) -> None:
        self.index.pack(packer)
        self.hash.pack(packer)

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> ArchivalProofNode:
        index = Uint32.unpack(unpacker)
        hash = Hash.unpack(unpacker)
        return cls(
            index=index,
            hash=hash,
        )

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> ArchivalProofNode:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> ArchivalProofNode:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash(
            (
                self.index,
                self.hash,
            )
        )

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.index == other.index and self.hash == other.hash

    def __repr__(self):
        out = [
            f"index={self.index}",
            f"hash={self.hash}",
        ]
        return f"<ArchivalProofNode [{', '.join(out)}]>"



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64

from xdrlib3 import Packer, Unpacker

from .operation_result_code import OperationResultCode
from .operation_result_tr import OperationResultTr

__all__ = ["OperationResult"]


class OperationResult:
    """
    XDR Source Code::

        union OperationResult switch (OperationResultCode code)
        {
        case opINNER:
            union switch (OperationType type)
            {
            case CREATE_ACCOUNT:
                CreateAccountResult createAccountResult;
            case PAYMENT:
                PaymentResult paymentResult;
            case PATH_PAYMENT_STRICT_RECEIVE:
                PathPaymentStrictReceiveResult pathPaymentStrictReceiveResult;
            case MANAGE_SELL_OFFER:
                ManageSellOfferResult manageSellOfferResult;
            case CREATE_PASSIVE_SELL_OFFER:
                ManageSellOfferResult createPassiveSellOfferResult;
            case SET_OPTIONS:
                SetOptionsResult setOptionsResult;
            case CHANGE_TRUST:
                ChangeTrustResult changeTrustResult;
            case ALLOW_TRUST:
                AllowTrustResult allowTrustResult;
            case ACCOUNT_MERGE:
                AccountMergeResult accountMergeResult;
            case INFLATION:
                InflationResult inflationResult;
            case MANAGE_DATA:
                ManageDataResult manageDataResult;
            case BUMP_SEQUENCE:
                BumpSequenceResult bumpSeqResult;
            case MANAGE_BUY_OFFER:
                ManageBuyOfferResult manageBuyOfferResult;
            case PATH_PAYMENT_STRICT_SEND:
                PathPaymentStrictSendResult pathPaymentStrictSendResult;
            case CREATE_CLAIMABLE_BALANCE:
                CreateClaimableBalanceResult createClaimableBalanceResult;
            case CLAIM_CLAIMABLE_BALANCE:
                ClaimClaimableBalanceResult claimClaimableBalanceResult;
            case BEGIN_SPONSORING_FUTURE_RESERVES:
                BeginSponsoringFutureReservesResult beginSponsoringFutureReservesResult;
            case END_SPONSORING_FUTURE_RESERVES:
                EndSponsoringFutureReservesResult endSponsoringFutureReservesResult;
            case REVOKE_SPONSORSHIP:
                RevokeSponsorshipResult revokeSponsorshipResult;
            case CLAWBACK:
                ClawbackResult clawbackResult;
            case CLAWBACK_CLAIMABLE_BALANCE:
                ClawbackClaimableBalanceResult clawbackClaimableBalanceResult;
            case SET_TRUST_LINE_FLAGS:
                SetTrustLineFlagsResult setTrustLineFlagsResult;
            case LIQUIDITY_POOL_DEPOSIT:
                LiquidityPoolDepositResult liquidityPoolDepositResult;
            case LIQUIDITY_POOL_WITHDRAW:
                LiquidityPoolWithdrawResult liquidityPoolWithdrawResult;
            case INVOKE_HOST_FUNCTION:
                InvokeHostFunctionResult invokeHostFunctionResult;
            case EXTEND_FOOTPRINT_TTL:
                ExtendFootprintTTLResult extendFootprintTTLResult;
            case RESTORE_FOOTPRINT:
                RestoreFootprintResult restoreFootprintResult;
            }
            tr;
        case opBAD_AUTH:
        case opNO_ACCOUNT:
        case opNOT_SUPPORTED:
        case opTOO_MANY_SUBENTRIES:
        case opEXCEEDED_WORK_LIMIT:
        case opTOO_MANY_SPONSORING:
            void;
        };
    """

    def __init__(
        self,
        code: OperationResultCode,
        tr: OperationResultTr = None,
    ) -> None:
        self.code = code
        self.tr = tr

    def pack(self, packer: Packer) -> None:
        self.code.pack(packer)
        if self.code == OperationResultCode.opINNER:
            if self.tr is None:
                raise ValueError("tr should not be None.")
            self.tr.pack(packer)
            return
        if self.code == OperationResultCode.opBAD_AUTH:
            return
        if self.code == OperationResultCode.opNO_ACCOUNT:
            return
        if self.code == OperationResultCode.opNOT_SUPPORTED:
            return
        if self.code == OperationResultCode.opTOO_MANY_SUBENTRIES:
            return
        if self.code == OperationResultCode.opEXCEEDED_WORK_LIMIT:
            return
        if self.code == OperationResultCode.opTOO_MANY_SPONSORING:
            return

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> OperationResult:
        code = OperationResultCode.unpack(unpacker)
        if code == OperationResultCode.opINNER:
            tr = OperationResultTr.unpack(unpacker)
            return cls(code=code, tr=tr)
        if code == OperationResultCode.opBAD_AUTH:
            return cls(code=code)
        if code == OperationResultCode.opNO_ACCOUNT:
            return cls(code=code)
        if code == OperationResultCode.opNOT_SUPPORTED:
            return cls(code=code)
        if code == OperationResultCode.opTOO_MANY_SUBENTRIES:
            return cls(code=code)
        if code == OperationResultCode.opEXCEEDED_WORK_LIMIT:
            return cls(code=code)
        if code == OperationResultCode.opTOO_MANY_SPONSORING:
            return cls(code=code)
        return cls(code=code)

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> OperationResult:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> OperationResult:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash(
            (
                self.code,
                self.tr,
            )
        )

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.code == other.code and self.tr == other.tr

    def __repr__(self):
        out = []
        out.append(f"code={self.code}")
        out.append(f"tr={self.tr}") if self.tr is not None else None
        return f"<OperationResult [{', '.join(out)}]>"



import abc
from typing import Union

from . import xdr as stellar_xdr
from .exceptions import MemoInvalidException
from .utils import hex_to_bytes

__all__ = ["Memo", "NoneMemo", "TextMemo", "IdMemo", "HashMemo", "ReturnHashMemo"]


class Memo(object, metaclass=abc.ABCMeta):
    """The :class:`Memo` object, which represents the base class for memos for
    use with Stellar transactions.

    The memo for a transaction contains optional extra information about the
    transaction taking place. It is the responsibility of the client to
    interpret this value.

    See the following implementations that serve a more practical use with the
    library:

    * :class:`NoneMemo` - No memo.
    * :class:`TextMemo` - A string encoded using either ASCII or UTF-8, up to 28-bytes long.
    * :class:`IdMemo` - A 64 bit unsigned integer.
    * :class:`HashMemo` - A 32 byte hash.
    * :class:`RetHashMemo` -  A 32 byte hash intended to be interpreted as the hash of the transaction the sender is refunding.

    See `Stellar's documentation on Transactions
    <https://developers.stellar.org/docs/glossary/transactions/#memo>`__
    for more information on how memos are used within transactions, as well as
    information on the available types of memos.

    """

    @abc.abstractmethod
    def to_xdr_object(self) -> stellar_xdr.Memo:
        """Creates an XDR Memo object that represents this :class:`Memo`."""

    @staticmethod
    def from_xdr_object(xdr_object: stellar_xdr.Memo) -> "Memo":
        """Returns an Memo object from XDR memo object."""

        xdr_types = {
            stellar_xdr.MemoType.MEMO_TEXT: TextMemo,
            stellar_xdr.MemoType.MEMO_ID: IdMemo,
            stellar_xdr.MemoType.MEMO_HASH: HashMemo,
            stellar_xdr.MemoType.MEMO_RETURN: ReturnHashMemo,
            stellar_xdr.MemoType.MEMO_NONE: NoneMemo,
        }

        memo_cls = xdr_types.get(xdr_object.type, NoneMemo)
        return memo_cls.from_xdr_object(xdr_object)  # type: ignore[attr-defined]

    @abc.abstractmethod
    def __hash__(self):
        pass  # pragma: no cover

    @abc.abstractmethod
    def __eq__(self, other: object) -> bool:
        pass  # pragma: no cover


class NoneMemo(Memo):
    """The :class:`NoneMemo`, which represents no memo for a transaction."""

    @classmethod
    def from_xdr_object(cls, xdr_object: stellar_xdr.Memo) -> "NoneMemo":
        """Returns an :class:`NoneMemo` object from XDR memo object."""

        return cls()

    def to_xdr_object(self) -> stellar_xdr.Memo:
        """Creates an XDR Memo object that represents this :class:`NoneMemo`."""
        return stellar_xdr.Memo(type=stellar_xdr.MemoType.MEMO_NONE)

    def __hash__(self):
        return hash(None)

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, self.__class__):
            return NotImplemented
        return True

    def __repr__(self):
        return "<NoneMemo>"


class TextMemo(Memo):
    """The :class:`TextMemo`, which represents ``MEMO_TEXT`` in a transaction.

    :param text: A string encoded using either ASCII or UTF-8, up to
        28-bytes long. Note, `text` can be anything,
        see `this issue <https://github.com/stellar/new-docs/issues/555>`__ for more information.
    :raises: :exc:`MemoInvalidException <stellar_sdk.exceptions.MemoInvalidException>`:
        if ``text`` is not a valid text memo.

    """

    def __init__(self, text: Union[str, bytes]) -> None:
        if not isinstance(text, bytes):
            text = bytes(text, encoding="utf-8")
        self.memo_text: bytes = text

        length = len(self.memo_text)
        if length > 28:
            raise MemoInvalidException(
                f"Text should be <= 28 bytes (ascii encoded), got {length} bytes."
            )

    @classmethod
    def from_xdr_object(cls, xdr_object: stellar_xdr.Memo) -> "TextMemo":
        """Returns an :class:`TextMemo` object from XDR memo object."""
        assert xdr_object.text is not None
        return cls(bytes(xdr_object.text))

    def to_xdr_object(self) -> stellar_xdr.Memo:
        """Creates an XDR Memo object that represents this :class:`TextMemo`."""
        return stellar_xdr.Memo(
            type=stellar_xdr.MemoType.MEMO_TEXT, text=self.memo_text
        )

    def __hash__(self):
        return hash(self.memo_text)

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.memo_text == other.memo_text

    def __repr__(self):
        return f"<TextMemo [memo={self.memo_text}]>"


class IdMemo(Memo):
    """The :class:`IdMemo` which represents ``MEMO_ID`` in a transaction.

    :param memo_id: A 64 bit unsigned integer.
    :raises:
        :exc:`MemoInvalidException <stellar_sdk.exceptions.MemoInvalidException>`:
        if ``id`` is not a valid id memo.

    """

    def __init__(self, memo_id: int) -> None:
        if memo_id < 0 or memo_id > 2**64 - 1:
            raise MemoInvalidException(
                "IdMemo is an unsigned 64-bit integer and the max valid value is 18446744073709551615."
            )
        self.memo_id: int = memo_id

    @classmethod
    def from_xdr_object(cls, xdr_object: stellar_xdr.Memo) -> "IdMemo":
        """Returns an :class:`IdMemo` object from XDR memo object."""
        assert xdr_object.id is not None
        return cls(xdr_object.id.uint64)

    def to_xdr_object(self) -> stellar_xdr.Memo:
        """Creates an XDR Memo object that represents this :class:`IdMemo`."""
        return stellar_xdr.Memo(
            type=stellar_xdr.MemoType.MEMO_ID, id=stellar_xdr.Uint64(self.memo_id)
        )

    def __hash__(self):
        return hash(self.memo_id)

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.memo_id == other.memo_id

    def __repr__(self):
        return f"<IdMemo [memo={self.memo_id}]>"


class HashMemo(Memo):
    """The :class:`HashMemo` which represents ``MEMO_HASH`` in a transaction.

    :param memo_hash: A 32 byte hash hex encoded string.
    :raises: :exc:`MemoInvalidException <stellar_sdk.exceptions.MemoInvalidException>`:
        if ``memo_hash`` is not a valid hash memo.
    """

    def __init__(self, memo_hash: Union[bytes, str]) -> None:
        memo_hash = hex_to_bytes(memo_hash)
        length = len(memo_hash)
        if length != 32:
            raise MemoInvalidException(
                f"The length of HashMemo should be 32 bytes, got {length} bytes."
            )

        self.memo_hash: bytes = memo_hash

    @classmethod
    def from_xdr_object(cls, xdr_object: stellar_xdr.Memo) -> "HashMemo":
        """Returns an :class:`HashMemo` object from XDR memo object."""
        assert xdr_object.hash is not None
        return cls(xdr_object.hash.hash)

    def to_xdr_object(self) -> stellar_xdr.Memo:
        """Creates an XDR Memo object that represents this :class:`HashMemo`."""
        return stellar_xdr.Memo(
            type=stellar_xdr.MemoType.MEMO_HASH, hash=stellar_xdr.Hash(self.memo_hash)
        )

    def __hash__(self):
        return hash(self.memo_hash)

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.memo_hash == other.memo_hash

    def __repr__(self):
        return f"<HashMemo [memo={self.memo_hash}]>"


class ReturnHashMemo(Memo):
    """The :class:`ReturnHashMemo` which represents ``MEMO_RETURN`` in a transaction.

    MEMO_RETURN is typically used with refunds/returns over the network - it is
    a 32 byte hash intended to be interpreted as the hash of the transaction
    the sender is refunding.

    :param memo_return: A 32 byte hash or hex encoded string intended to be interpreted as the
        hash of the transaction the sender is refunding.
    :raises: :exc:`MemoInvalidException <stellar_sdk.exceptions.MemoInvalidException>`:
        if ``memo_return`` is not a valid return hash memo.
    """

    def __init__(self, memo_return: Union[bytes, str]) -> None:
        memo_return = hex_to_bytes(memo_return)
        length = len(memo_return)
        if length != 32:
            raise MemoInvalidException(
                f"The length of ReturnHashMemo should be 32 bytes, got {length} bytes."
            )

        self.memo_return: bytes = memo_return

    @classmethod
    def from_xdr_object(cls, xdr_object: stellar_xdr.Memo) -> "ReturnHashMemo":
        """Returns an :class:`ReturnHashMemo` object from XDR memo object."""
        assert xdr_object.ret_hash is not None
        return cls(xdr_object.ret_hash.hash)

    def to_xdr_object(self) -> stellar_xdr.Memo:
        """Creates an XDR Memo object that represents this :class:`ReturnHashMemo`."""
        return stellar_xdr.Memo(
            type=stellar_xdr.MemoType.MEMO_RETURN,
            ret_hash=stellar_xdr.Hash(self.memo_return),
        )

    def __hash__(self):
        return hash(self.memo_return)

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.memo_return == other.memo_return

    def __repr__(self):
        return f"<ReturnHashMemo [memo={self.memo_return}]>"



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64
from typing import List

from xdrlib3 import Packer, Unpacker

from .node_id import NodeID
from .uint32 import Uint32

__all__ = ["SCPQuorumSet"]


class SCPQuorumSet:
    """
    XDR Source Code::

        struct SCPQuorumSet
        {
            uint32 threshold;
            NodeID validators<>;
            SCPQuorumSet innerSets<>;
        };
    """

    def __init__(
        self,
        threshold: Uint32,
        validators: List[NodeID],
        inner_sets: List["SCPQuorumSet"],
    ) -> None:
        _expect_max_length = 4294967295
        if validators and len(validators) > _expect_max_length:
            raise ValueError(
                f"The maximum length of `validators` should be {_expect_max_length}, but got {len(validators)}."
            )
        _expect_max_length = 4294967295
        if inner_sets and len(inner_sets) > _expect_max_length:
            raise ValueError(
                f"The maximum length of `inner_sets` should be {_expect_max_length}, but got {len(inner_sets)}."
            )
        self.threshold = threshold
        self.validators = validators
        self.inner_sets = inner_sets

    def pack(self, packer: Packer) -> None:
        self.threshold.pack(packer)
        packer.pack_uint(len(self.validators))
        for validators_item in self.validators:
            validators_item.pack(packer)
        packer.pack_uint(len(self.inner_sets))
        for inner_sets_item in self.inner_sets:
            inner_sets_item.pack(packer)

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> SCPQuorumSet:
        threshold = Uint32.unpack(unpacker)
        length = unpacker.unpack_uint()
        validators = []
        for _ in range(length):
            validators.append(NodeID.unpack(unpacker))
        length = unpacker.unpack_uint()
        inner_sets = []
        for _ in range(length):
            inner_sets.append(SCPQuorumSet.unpack(unpacker))
        return cls(
            threshold=threshold,
            validators=validators,
            inner_sets=inner_sets,
        )

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> SCPQuorumSet:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> SCPQuorumSet:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash(
            (
                self.threshold,
                self.validators,
                self.inner_sets,
            )
        )

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return (
            self.threshold == other.threshold
            and self.validators == other.validators
            and self.inner_sets == other.inner_sets
        )

    def __repr__(self):
        out = [
            f"threshold={self.threshold}",
            f"validators={self.validators}",
            f"inner_sets={self.inner_sets}",
        ]
        return f"<SCPQuorumSet [{', '.join(out)}]>"



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64

from xdrlib3 import Packer, Unpacker

from .ledger_close_value_signature import LedgerCloseValueSignature
from .stellar_value_type import StellarValueType

__all__ = ["StellarValueExt"]


class StellarValueExt:
    """
    XDR Source Code::

        union switch (StellarValueType v)
            {
            case STELLAR_VALUE_BASIC:
                void;
            case STELLAR_VALUE_SIGNED:
                LedgerCloseValueSignature lcValueSignature;
            }
    """

    def __init__(
        self,
        v: StellarValueType,
        lc_value_signature: LedgerCloseValueSignature = None,
    ) -> None:
        self.v = v
        self.lc_value_signature = lc_value_signature

    def pack(self, packer: Packer) -> None:
        self.v.pack(packer)
        if self.v == StellarValueType.STELLAR_VALUE_BASIC:
            return
        if self.v == StellarValueType.STELLAR_VALUE_SIGNED:
            if self.lc_value_signature is None:
                raise ValueError("lc_value_signature should not be None.")
            self.lc_value_signature.pack(packer)
            return

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> StellarValueExt:
        v = StellarValueType.unpack(unpacker)
        if v == StellarValueType.STELLAR_VALUE_BASIC:
            return cls(v=v)
        if v == StellarValueType.STELLAR_VALUE_SIGNED:
            lc_value_signature = LedgerCloseValueSignature.unpack(unpacker)
            return cls(v=v, lc_value_signature=lc_value_signature)
        return cls(v=v)

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> StellarValueExt:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> StellarValueExt:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash(
            (
                self.v,
                self.lc_value_signature,
            )
        )

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.v == other.v and self.lc_value_signature == other.lc_value_signature

    def __repr__(self):
        out = []
        out.append(f"v={self.v}")
        (
            out.append(f"lc_value_signature={self.lc_value_signature}")
            if self.lc_value_signature is not None
            else None
        )
        return f"<StellarValueExt [{', '.join(out)}]>"



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64
from typing import List

from xdrlib3 import Packer, Unpacker

from .hash import Hash
from .transaction_envelope import TransactionEnvelope

__all__ = ["TransactionSet"]


class TransactionSet:
    """
    XDR Source Code::

        struct TransactionSet
        {
            Hash previousLedgerHash;
            TransactionEnvelope txs<>;
        };
    """

    def __init__(
        self,
        previous_ledger_hash: Hash,
        txs: List[TransactionEnvelope],
    ) -> None:
        _expect_max_length = 4294967295
        if txs and len(txs) > _expect_max_length:
            raise ValueError(
                f"The maximum length of `txs` should be {_expect_max_length}, but got {len(txs)}."
            )
        self.previous_ledger_hash = previous_ledger_hash
        self.txs = txs

    def pack(self, packer: Packer) -> None:
        self.previous_ledger_hash.pack(packer)
        packer.pack_uint(len(self.txs))
        for txs_item in self.txs:
            txs_item.pack(packer)

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> TransactionSet:
        previous_ledger_hash = Hash.unpack(unpacker)
        length = unpacker.unpack_uint()
        txs = []
        for _ in range(length):
            txs.append(TransactionEnvelope.unpack(unpacker))
        return cls(
            previous_ledger_hash=previous_ledger_hash,
            txs=txs,
        )

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> TransactionSet:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> TransactionSet:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash(
            (
                self.previous_ledger_hash,
                self.txs,
            )
        )

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return (
            self.previous_ledger_hash == other.previous_ledger_hash
            and self.txs == other.txs
        )

    def __repr__(self):
        out = [
            f"previous_ledger_hash={self.previous_ledger_hash}",
            f"txs={self.txs}",
        ]
        return f"<TransactionSet [{', '.join(out)}]>"



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64
from enum import IntEnum

from xdrlib3 import Packer, Unpacker

__all__ = ["OperationResultCode"]


class OperationResultCode(IntEnum):
    """
    XDR Source Code::

        enum OperationResultCode
        {
            opINNER = 0, // inner object result is valid

            opBAD_AUTH = -1,            // too few valid signatures / wrong network
            opNO_ACCOUNT = -2,          // source account was not found
            opNOT_SUPPORTED = -3,       // operation not supported at this time
            opTOO_MANY_SUBENTRIES = -4, // max number of subentries already reached
            opEXCEEDED_WORK_LIMIT = -5, // operation did too much work
            opTOO_MANY_SPONSORING = -6  // account is sponsoring too many entries
        };
    """

    opINNER = 0
    opBAD_AUTH = -1
    opNO_ACCOUNT = -2
    opNOT_SUPPORTED = -3
    opTOO_MANY_SUBENTRIES = -4
    opEXCEEDED_WORK_LIMIT = -5
    opTOO_MANY_SPONSORING = -6

    def pack(self, packer: Packer) -> None:
        packer.pack_int(self.value)

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> OperationResultCode:
        value = unpacker.unpack_int()
        return cls(value)

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> OperationResultCode:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> OperationResultCode:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64
from enum import IntEnum

from xdrlib3 import Packer, Unpacker

__all__ = ["LedgerEntryType"]


class LedgerEntryType(IntEnum):
    """
    XDR Source Code::

        enum LedgerEntryType
        {
            ACCOUNT = 0,
            TRUSTLINE = 1,
            OFFER = 2,
            DATA = 3,
            CLAIMABLE_BALANCE = 4,
            LIQUIDITY_POOL = 5,
            CONTRACT_DATA = 6,
            CONTRACT_CODE = 7,
            CONFIG_SETTING = 8,
            TTL = 9
        };
    """

    ACCOUNT = 0
    TRUSTLINE = 1
    OFFER = 2
    DATA = 3
    CLAIMABLE_BALANCE = 4
    LIQUIDITY_POOL = 5
    CONTRACT_DATA = 6
    CONTRACT_CODE = 7
    CONFIG_SETTING = 8
    TTL = 9

    def pack(self, packer: Packer) -> None:
        packer.pack_int(self.value)

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> LedgerEntryType:
        value = unpacker.unpack_int()
        return cls(value)

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> LedgerEntryType:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> LedgerEntryType:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64
from enum import IntEnum

from xdrlib3 import Packer, Unpacker

__all__ = ["InvokeHostFunctionResultCode"]


class InvokeHostFunctionResultCode(IntEnum):
    """
    XDR Source Code::

        enum InvokeHostFunctionResultCode
        {
            // codes considered as "success" for the operation
            INVOKE_HOST_FUNCTION_SUCCESS = 0,

            // codes considered as "failure" for the operation
            INVOKE_HOST_FUNCTION_MALFORMED = -1,
            INVOKE_HOST_FUNCTION_TRAPPED = -2,
            INVOKE_HOST_FUNCTION_RESOURCE_LIMIT_EXCEEDED = -3,
            INVOKE_HOST_FUNCTION_ENTRY_ARCHIVED = -4,
            INVOKE_HOST_FUNCTION_INSUFFICIENT_REFUNDABLE_FEE = -5
        };
    """

    INVOKE_HOST_FUNCTION_SUCCESS = 0
    INVOKE_HOST_FUNCTION_MALFORMED = -1
    INVOKE_HOST_FUNCTION_TRAPPED = -2
    INVOKE_HOST_FUNCTION_RESOURCE_LIMIT_EXCEEDED = -3
    INVOKE_HOST_FUNCTION_ENTRY_ARCHIVED = -4
    INVOKE_HOST_FUNCTION_INSUFFICIENT_REFUNDABLE_FEE = -5

    def pack(self, packer: Packer) -> None:
        packer.pack_int(self.value)

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> InvokeHostFunctionResultCode:
        value = unpacker.unpack_int()
        return cls(value)

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> InvokeHostFunctionResultCode:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> InvokeHostFunctionResultCode:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)



from xdrlib3 import Packer, Unpacker

__all__ = [
    "Integer",
    "UnsignedInteger",
    "Float",
    "Double",
    "Hyper",
    "UnsignedHyper",
    "Boolean",
    "String",
    "Opaque",
]


class Integer:
    def __init__(self, value: int) -> None:
        self.value = value

    def pack(self, packer: Packer) -> None:
        packer.pack_int(self.value)

    @staticmethod
    def unpack(unpacker: Unpacker) -> int:
        return unpacker.unpack_int()

    def __hash__(self):
        return hash(self.value)

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.value == other.value

    def __repr__(self):
        return f"<Integer [value={self.value}]>"


class UnsignedInteger:
    def __init__(self, value: int) -> None:
        self.value = value

    def pack(self, packer: Packer) -> None:
        packer.pack_uint(self.value)

    @staticmethod
    def unpack(unpacker: Unpacker) -> int:
        return unpacker.unpack_uint()

    def __hash__(self):
        return hash(self.value)

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.value == other.value

    def __repr__(self):
        return f"<UnsignedInteger [value={self.value}]>"


class Float:
    def __init__(self, value: float) -> None:
        self.value = value

    def pack(self, packer: Packer) -> None:
        packer.pack_float(self.value)

    @staticmethod
    def unpack(unpacker: Unpacker) -> float:
        return unpacker.unpack_float()

    def __hash__(self):
        return hash(self.value)

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.value == other.value

    def __repr__(self):
        return f"<Float [value={self.value}]>"


class Double:
    def __init__(self, value: float) -> None:
        self.value = value

    def pack(self, packer: Packer) -> None:
        packer.pack_double(self.value)

    @staticmethod
    def unpack(unpacker: Unpacker) -> float:
        return unpacker.unpack_double()

    def __hash__(self):
        return hash(self.value)

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.value == other.value

    def __repr__(self):
        return f"<Double [value={self.value}]>"


class Hyper:
    def __init__(self, value: int) -> None:
        self.value = value

    def pack(self, packer: Packer) -> None:
        packer.pack_hyper(self.value)

    @staticmethod
    def unpack(unpacker: Unpacker) -> int:
        return unpacker.unpack_hyper()

    def __hash__(self):
        return hash(self.value)

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.value == other.value

    def __repr__(self):
        return f"<Hyper [value={self.value}]>"


class UnsignedHyper:
    def __init__(self, value: int) -> None:
        self.value = value

    def pack(self, packer: Packer) -> None:
        packer.pack_uhyper(self.value)

    @staticmethod
    def unpack(unpacker: Unpacker) -> int:
        return unpacker.unpack_uhyper()

    def __hash__(self):
        return hash(self.value)

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.value == other.value

    def __repr__(self):
        return f"<UnsignedHyper [value={self.value}]>"


class Boolean:
    def __init__(self, value: bool) -> None:
        self.value = value

    def pack(self, packer: Packer) -> None:
        packer.pack_bool(self.value)

    @staticmethod
    def unpack(unpacker: Unpacker) -> bool:
        return unpacker.unpack_bool()

    def __hash__(self):
        return hash(self.value)

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.value == other.value

    def __repr__(self):
        return f"<Boolean [value={self.value}]>"


class String:
    def __init__(self, value: bytes, size: int) -> None:
        if len(value) > size:
            raise ValueError(
                f"The maximum length of `value` should be {size}, but got {len(value)}."
            )

        self.value = value
        self.size = len(value)

    def pack(self, packer: Packer) -> None:
        packer.pack_uint(len(self.value))
        packer.pack_fopaque(len(self.value), self.value)

    @staticmethod
    def unpack(unpacker: Unpacker) -> bytes:
        size = unpacker.unpack_uint()
        return unpacker.unpack_fopaque(size)

    def __hash__(self):
        return hash((self.value, self.size))

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.value == other.value and self.size == other.size

    def __repr__(self):
        return f"<String [value={self.value}, size={self.size}]>"


class Opaque:
    def __init__(self, value: bytes, size: int, fixed: bool) -> None:
        if fixed:
            if len(value) != size:
                raise ValueError(
                    f"The length of `value` should be {size}, but got {len(value)}."
                )
        else:
            if len(value) > size:
                raise ValueError(
                    f"The maximum length of `value` should be {size}, but got {len(value)}."
                )

        self.value = value
        self.fixed = fixed
        self.size = len(value)

    def pack(self, packer: Packer) -> None:
        if not self.fixed:
            size = len(self.value)
            packer.pack_uint(size)
        else:
            size = self.size
        packer.pack_fopaque(size, self.value)

    @staticmethod
    def unpack(unpacker: Unpacker, size: int, fixed: bool) -> bytes:
        if not fixed:
            size = unpacker.unpack_uint()
        return unpacker.unpack_fopaque(size)

    def __hash__(self):
        return hash((self.value, self.size, self.fixed))

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, self.__class__):
            return NotImplemented
        return (
            self.value == other.value
            and self.fixed == other.fixed
            and self.size == other.size
        )

    def __repr__(self):
        return f"<Opaque [value={self.value}, fixed={self.fixed}, size={self.size}]>"



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64
from typing import List

from xdrlib3 import Packer, Unpacker

from .sc_spec_type_def import SCSpecTypeDef

__all__ = ["SCSpecTypeTuple"]


class SCSpecTypeTuple:
    """
    XDR Source Code::

        struct SCSpecTypeTuple
        {
            SCSpecTypeDef valueTypes<12>;
        };
    """

    def __init__(
        self,
        value_types: List[SCSpecTypeDef],
    ) -> None:
        _expect_max_length = 12
        if value_types and len(value_types) > _expect_max_length:
            raise ValueError(
                f"The maximum length of `value_types` should be {_expect_max_length}, but got {len(value_types)}."
            )
        self.value_types = value_types

    def pack(self, packer: Packer) -> None:
        packer.pack_uint(len(self.value_types))
        for value_types_item in self.value_types:
            value_types_item.pack(packer)

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> SCSpecTypeTuple:
        length = unpacker.unpack_uint()
        value_types = []
        for _ in range(length):
            value_types.append(SCSpecTypeDef.unpack(unpacker))
        return cls(
            value_types=value_types,
        )

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> SCSpecTypeTuple:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> SCSpecTypeTuple:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash((self.value_types,))

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.value_types == other.value_types

    def __repr__(self):
        out = [
            f"value_types={self.value_types}",
        ]
        return f"<SCSpecTypeTuple [{', '.join(out)}]>"



"""
SEP: 0005
Title: Key Derivation Methods for Stellar Accounts
"""

import hashlib
import hmac
import os
import struct
from enum import Enum, unique
from typing import Union

from mnemonic import Mnemonic
from mnemonic.mnemonic import PBKDF2_ROUNDS


@unique
class Language(Enum):
    """The type of language supported by the mnemonic."""

    JAPANESE = "japanese"
    FRENCH = "french"
    ENGLISH = "english"
    SPANISH = "spanish"
    ITALIAN = "italian"
    KOREAN = "korean"
    CHINESE_SIMPLIFIED = "chinese_simplified"
    CHINESE_TRADITIONAL = "chinese_traditional"


class StellarMnemonic(Mnemonic):
    """Please use :func:`stellar_sdk.keypair.Keypair.generate_mnemonic_phrase`
    and :func:`stellar_sdk.keypair.Keypair.from_mnemonic_phrase`
    """

    STELLAR_ACCOUNT_PATH_FORMAT = "m/44'/148'/%d'"
    FIRST_HARDENED_INDEX = 0x80000000
    SEED_MODIFIER = b"ed25519 seed"

    def __init__(self, language: Union[str, Language] = Language.ENGLISH) -> None:
        if isinstance(language, Language):
            language = language.value
        else:
            if language not in set(item.value for item in Language):
                raise ValueError("This language is not supported.")

        super().__init__(language)

    def to_seed(self, mnemonic: str, passphrase: str = "", index: int = 0) -> bytes:  # type: ignore[override]
        if not self.check(mnemonic):
            raise ValueError(
                "Invalid mnemonic, please check if the mnemonic is correct, "
                "or if the language is set correctly."
            )
        mnemonic = self.normalize_string(mnemonic)
        passphrase = self.normalize_string(passphrase)
        passphrase = "mnemonic" + passphrase
        mnemonic_bytes = mnemonic.encode("utf-8")
        passphrase_bytes = passphrase.encode("utf-8")
        stretched = hashlib.pbkdf2_hmac(
            "sha512", mnemonic_bytes, passphrase_bytes, PBKDF2_ROUNDS
        )
        return self.derive(stretched[:64], index)

    def generate(self, strength: int = 128) -> str:
        if strength not in (128, 160, 192, 224, 256):
            raise ValueError(
                f"Strength should be one of the following (128, 160, 192, 224, 256), but it is not ({strength})."
            )
        return self.to_mnemonic(os.urandom(strength // 8))

    @staticmethod
    def derive(seed: bytes, index: int) -> bytes:
        # References https://github.com/satoshilabs/slips/blob/master/slip-0010.md
        master_hmac = hmac.new(StellarMnemonic.SEED_MODIFIER, digestmod=hashlib.sha512)
        master_hmac.update(seed)
        il = master_hmac.digest()[:32]
        ir = master_hmac.digest()[32:]
        path = StellarMnemonic.STELLAR_ACCOUNT_PATH_FORMAT % index
        for x in path.split("/")[1:]:
            data = (
                struct.pack("x")
                + il
                + struct.pack(">I", StellarMnemonic.FIRST_HARDENED_INDEX + int(x[:-1]))
            )
            i = hmac.new(ir, digestmod=hashlib.sha512)
            i.update(data)
            il = i.digest()[:32]
            ir = i.digest()[32:]
        return il



from ...asset import Asset
from ...call_builder.base.base_call_builder import BaseCallBuilder

__all__ = ["BaseTradeAggregationsCallBuilder"]


class BaseTradeAggregationsCallBuilder(BaseCallBuilder):
    """Creates a new :class:`TradeAggregationsCallBuilder` pointed to server defined by horizon_url.

    Trade Aggregations facilitate efficient gathering of historical trade data.

    See `List Trade Aggregations <https://developers.stellar.org/api/aggregations/trade-aggregations/list/>`__ for more information.

    :param base: base asset
    :param counter: counter asset
    :param resolution: segment duration as millis since epoch. *Supported values
        are 1 minute (60000), 5 minutes (300000), 15 minutes (900000),
        1 hour (3600000), 1 day (86400000) and 1 week (604800000).*
    :param start_time: lower time boundary represented as millis since epoch
    :param end_time: upper time boundary represented as millis since epoch
    :param offset: segments can be offset using this parameter.
        Expressed in milliseconds. *Can only be used if the resolution is greater than 1 hour.
        Value must be in whole hours, less than the provided resolution, and less than 24 hours.*
    :param horizon_url: Horizon server URL.
    """

    def __init__(
        self,
        base: Asset,
        counter: Asset,
        resolution: int,
        start_time: int = None,
        end_time: int = None,
        offset: int = None,
        **kwargs,
    ) -> None:
        super().__init__(**kwargs)
        self.endpoint: str = "trade_aggregations"

        if not self._is_valid_resolution(resolution):
            raise ValueError(f"Invalid resolution: {resolution}")

        if offset and not self._is_valid_offset(offset, resolution):
            raise ValueError(f"Invalid offset: {offset}")

        params = {
            "base_asset_type": base.type,
            "base_asset_code": None if base.is_native() else base.code,
            "base_asset_issuer": base.issuer,
            "counter_asset_type": counter.type,
            "counter_asset_code": None if counter.is_native() else counter.code,
            "counter_asset_issuer": counter.issuer,
            "start_time": start_time,
            "end_time": end_time,
            "resolution": resolution,
            "offset": offset,
        }
        self._add_query_params(params)

    def _is_valid_offset(self, offset: int, resolution: int) -> bool:
        """
        :param offset: Time offset in milliseconds
        :param resolution: Trade data resolution in milliseconds
        :return: `True` if the offset is allowed
        """
        hour = 3600000
        invalid = offset > resolution or offset >= 24 * hour or offset % hour != 0
        return not invalid

    def _is_valid_resolution(self, resolution: int) -> bool:
        """
        :param resolution: Trade data resolution in milliseconds
        :return: `True` if the resolution is allowed
        """
        allowed_resolutions = (60000, 300000, 900000, 3600000, 86400000, 604800000)
        if resolution in allowed_resolutions:
            return True
        return False



from typing import Any, Dict, List, Optional, Union

from .muxed_account import MuxedAccount
from .sep.ed25519_public_key_signer import Ed25519PublicKeySigner

__all__ = ["Account"]


class Account:
    """The :class:`Account` object represents a single
    account on the Stellar network and its sequence number.

    Account tracks the sequence number as it is used
    by :class:`TransactionBuilder <stellar_sdk.transaction_builder.TransactionBuilder>`.

    Normally, you can get an :class:`Account` instance through :func:`stellar_sdk.server.Server.load_account`
    or :func:`stellar_sdk.server_async.ServerAsync.load_account`.

    An example::

        from stellar_sdk import Keypair, Server

        server = Server(horizon_url="https://horizon-testnet.stellar.org")
        source = Keypair.from_secret("SBFZCHU5645DOKRWYBXVOXY2ELGJKFRX6VGGPRYUWHQ7PMXXJNDZFMKD")
        # `account` can also be a muxed account
        source_account = server.load_account(account=source.public_key)

    See `Accounts <https://developers.stellar.org/docs/glossary/accounts/>`__ for
    more information.

    :param account: Account Id of the
        account (ex. ``"GB3KJPLFUYN5VL6R3GU3EGCGVCKFDSD7BEDX42HWG5BWFKB3KQGJJRMA"``)
        or muxed account (ex. ``"MBZSQ3YZMZEWL5ZRCEQ5CCSOTXCFCMKDGFFP4IEQN2KN6LCHCLI46AAAAAAAAAAE2L2QE"``)
    :param sequence: Current sequence number of the account.
    :param raw_data: Raw horizon response data.
    """

    def __init__(
        self,
        account: Union[str, MuxedAccount],
        sequence: int,
        raw_data: Dict[str, Any] = None,
    ) -> None:
        if isinstance(account, str):
            self.account: MuxedAccount = MuxedAccount.from_account(account)
        else:
            self.account = account
        self.sequence: int = sequence
        self.raw_data: Optional[Dict[str, Any]] = raw_data

    @property
    def universal_account_id(self) -> str:
        """Get the universal account id,
        if `account` is ed25519 public key, it will return ed25519
        public key (ex. ``"GDGQVOKHW4VEJRU2TETD6DBRKEO5ERCNF353LW5WBFW3JJWQ2BRQ6KDD"``),
        otherwise it will return muxed
        account (ex. ``"MAAAAAAAAAAAJURAAB2X52XFQP6FBXLGT6LWOOWMEXWHEWBDVRZ7V5WH34Y22MPFBHUHY"``)
        """
        return self.account.universal_account_id

    def increment_sequence_number(self) -> None:
        """Increments sequence number in this object by one."""
        self.sequence += 1

    @property
    def thresholds(self):
        if self.raw_data is None:
            raise ValueError('"raw_data" is None, unable to get thresholds from it.')

        return Thresholds(
            self.raw_data["thresholds"]["low_threshold"],
            self.raw_data["thresholds"]["med_threshold"],
            self.raw_data["thresholds"]["high_threshold"],
        )

    def load_ed25519_public_key_signers(self) -> List[Ed25519PublicKeySigner]:
        """Load ed25519 public key signers."""
        if self.raw_data is None:
            raise ValueError('"raw_data" is None, unable to get signers from it.')

        signers = self.raw_data["signers"]
        ed25519_public_key_signers = []
        for signer in signers:
            if signer["type"] == "ed25519_public_key":
                ed25519_public_key_signers.append(
                    Ed25519PublicKeySigner(signer["key"], signer["weight"])
                )
        return ed25519_public_key_signers

    def __hash__(self):
        return hash((self.account, self.sequence, self.raw_data))

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.account == other.account and self.sequence == other.sequence

    def __repr__(self):
        return f"<Account [account={self.account}, sequence={self.sequence}]>"


class Thresholds:
    def __init__(
        self, low_threshold: int, med_threshold: int, high_threshold: int
    ) -> None:
        self.low_threshold = low_threshold
        self.med_threshold = med_threshold
        self.high_threshold = high_threshold

    def __hash__(self):
        return hash((self.low_threshold, self.med_threshold, self.high_threshold))

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, self.__class__):
            return NotImplemented
        return (
            self.low_threshold == other.low_threshold
            and self.med_threshold == other.med_threshold
            and self.high_threshold == other.high_threshold
        )

    def __repr__(self):
        return (
            f"<Thresholds [low_threshold={self.low_threshold}, med_threshold={self.med_threshold}, "
            f"high_threshold={self.high_threshold}]>"
        )
